/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * gui-transfer.h: Wrapper around GnomeTransfer that does blocking transfers
 * and handles caching
 *
 * Copyright (C) 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GUI_TRANSFER_H
#define _GUI_TRANSFER_H

#include <gtk/gtk.h>
#include <libsoup/soup.h>

#define GUI_TYPE_TRANSFER                (gui_transfer_get_type())
#define GUI_TRANSFER(obj)                (GTK_CHECK_CAST((obj), \
                                          GUI_TYPE_TRANSFER, GUITransfer))
#define GUI_TRANSFER_CLASS(klass)        (GTK_CHECK_CLASS_CAST((klass), \
                                          GUI_TYPE_TRANSFER, GUITransferClass))
#define GUI_IS_TRANSFER(obj)             (GTK_CHECK_TYPE((obj), \
                                          GUI_TYPE_TRANSFER))
#define GUI_IS_TRANSFER_CLASS(klass)     (GTK_CHECK_CLASS_TYPE((klass), \
                                          GUI_TYPE_TRANSFER))
#define GUI_TRANSFER_GET_CLASS(obj)      (GUI_TRANSFER_CLASS( \
                                          GTK_OBJECT(obj)->klass))

typedef   enum _GUITransferFlags          GUITransferFlags;
typedef   enum _GUITransferError          GUITransferError;
typedef   enum _GUITransferFilenameFormat GUITransferFilenameFormat;
typedef struct _GUITransfer               GUITransfer;
typedef struct _GUITransferClass          GUITransferClass;

enum _GUITransferFlags {
    GUI_TRANSFER_FLAGS_NONE           = 0,
    GUI_TRANSFER_FLAGS_BLOCK          = 1 << 0,
    GUI_TRANSFER_FLAGS_DONT_CACHE     = 1 << 1,
    GUI_TRANSFER_FLAGS_FORCE_CACHE    = 1 << 2,
    GUI_TRANSFER_FLAGS_FLUSH_MEMORY   = 1 << 3,
    GUI_TRANSFER_FLAGS_RESUME_PARTIAL = 1 << 4
};

enum _GUITransferError {
    GUI_TRANSFER_ERROR_NONE = 0,
    GUI_TRANSFER_ERROR_CANCELLED,
    GUI_TRANSFER_ERROR_CANT_CONNECT,
    GUI_TRANSFER_ERROR_FILE_NOT_FOUND,
    GUI_TRANSFER_ERROR_IO,
    GUI_TRANSFER_ERROR_CANT_AUTHENTICATE,
    GUI_TRANSFER_ERROR_INVALID_URI
};

struct _GUITransfer {
    GtkObject parent;

    /* GUITransferFlags */
    GUITransferFlags flags;

    GUITransferError error;
    char *error_string;

    /* Byte array of the data if we're blocking */
    GByteArray *data;

    /* URL of the file */
    char *url;
    /* Local filename */
    char *filename;

    /* Total file size */
    int file_size;
    /* Size downloaded so far */
    int trans_size;

    GtkWidget *progress_bar;
    GtkWidget *label;

    gboolean partial;

    /* Protocol-specific data */
    gpointer proto_data;

    /* Can this protocol-type be cached? */
    gboolean cacheable;

    /* The cache for this transfer */
    GUICache *cache;

    gboolean cached;
    gboolean aborted;
    gboolean paused;
    int offset;
    GTimer *timer;

    /* Hack for HTTP: Content-Range overrides Content-Length in determining
       file size. */
    gboolean content_range_set;
};

struct _GUITransferClass {
    GtkObjectClass parent_class;

    /* Signals */

    void (*file_data)(GUITransfer *, char *buffer, int size);
    void (*file_done)(GUITransfer *);
};

GtkType gui_transfer_get_type(void);
GUITransfer *gui_transfer_new(GUITransferFlags flags, GUICache *cache);
void gui_transfer_set_flags(GUITransfer *t, GUITransferFlags flags);
GByteArray *gui_transfer_begin(GUITransfer *t, const char *url);
void gui_transfer_abort(GUITransfer *t);
void gui_transfer_pause(GUITransfer *t);
void gui_transfer_resume(GUITransfer *t);
void gui_transfer_set_proxy_url(GUITransfer *t, const char *url);
void gui_transfer_set_progress_bar(GUITransfer *t, GtkProgress *progress_bar);
void gui_transfer_set_progress_label(GUITransfer *t, GtkLabel *lael);
GSList *gui_transfer_get_current_transfers(void);
GUITransferError gui_transfer_get_error(GUITransfer *t);
const char *gui_transfer_get_error_string(GUITransfer *t);

gboolean gui_transfer_error_box(GUITransfer *t, const char *message);

void rc_dump_transfer_info(SoupMessage *message);

#endif /* _GUI_TRANSFER_H */
